<?php
/*
schreinerwissen. auftragsverwaltung 
Copyright (C) 2011  Carsten Hilbert
c-hilbert@web.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/**
 *
 * @param type $file
 * @return type 
 */
function getQueriesFromFile($file)
{
    // import file line by line
    // and filter (remove) those lines, beginning with an sql comment token
    $file = array_filter(file($file),
                         create_function('$line',
                                         'return strpos(ltrim($line), "--") !== 0;'));
    // this is a list of SQL commands, which are allowed to follow a semicolon
    $keywords = array('ALTER', 'CREATE', 'DELETE', 'DROP', 'INSERT', 'REPLACE', 'SELECT', 'SET',
                      'TRUNCATE', 'UPDATE', 'USE');
    // create the regular expression
    $regexp = sprintf('/\s*;\s*(?=(%s)\b)/s', implode('|', $keywords));
    // split there
    $splitter = preg_split($regexp, implode("\r\n", $file));
    // remove trailing semicolon or whitespaces
    $splitter = array_map(create_function('$line',
                                          'return preg_replace("/[\s;]*$/", "", $line);'),
                          $splitter);
    // remove empty lines
    return array_filter($splitter, create_function('$line', 'return !empty($line);'));
} 



/**
 * Make from 5 "00005" or from 568 "00568"
 * 
 * @param string $no
 * @return string 
 */
function get_number_with_5_chars($no) {
    
    #$arr_no = explode(',', $no);
    $number_is = strlen($no);
    $length = 5;
    $str_zero = '';
    $zeros = $length - $number_is;
    for($i=0; $i<$zeros; $i++) {
        $str_zero .= '0';
    }
    $no_new = $str_zero . $no;
    return $no_new;
    
    
    
}

/**
 * Functions for schreinerwissen faktura
 * 
 * @since 2011
 */

/**
 *
 * @param string $dir directory path
 * @param integer $pos tabstop 
 */
function show_dir( $dir, $pos=2 )
{

    if ( $pos == 2 ) {
        echo "<pre>";
    }
 
    $handle = @opendir( $dir );
    if ( is_resource( $handle ) ) {
        while ( ( $file = readdir( $handle ) ) !== false ) {
            if ( preg_match( '~^\.{1,2}$~', $file ) ) {
                continue;
            }
            $file_link = '<li><a href="'.$dir.$file.'">'.$file.'</a></li>';
 
            if ( is_dir( $dir.$file ) ) {
                printf( "% ".$pos."s <b>%s</b>\n", '|-', $file );
                show_dir( $dir.$file.'/', $pos + 3 );
            } else {
                printf( "% ".$pos."s %s\n", '|-', $file_link );
            }
        }
        closedir($handle);
    }
 
    if ( $pos == 2 ) {
        echo "</pre>";
    }
}





/**
* directory_list
* return an array containing optionally all files, only directiories or only files at a file system path
* @author     cgray The Metamedia Corporation www.metamedia.us
*
* @param    $base_path         string    either absolute or relative path
* @param    $filter_dir        boolean    Filter directories from result (ignored except in last directory if $recursive is true)
* @param    $filter_files    boolean    Filter files from result
* @param    $exclude        string    Pipe delimited string of files to always ignore
* @param    $recursive        boolean    Descend directory to the bottom?
* @return    $result_list    array    Nested array or false
* @access public
* @license    GPL v3
*/
function directory_list($directory_base_path, $filter_dir = false, $filter_files = false, $exclude = ".|..|.DS_Store|.svn", $recursive = true){
    $directory_base_path = rtrim($directory_base_path, "/") . "/";

    if (!is_dir($directory_base_path)){
        error_log(__FUNCTION__ . "File at: $directory_base_path is not a directory.");
        return false;
    }

    $result_list = array();
    $exclude_array = explode("|", $exclude);

    if (!$folder_handle = opendir($directory_base_path)) {
        error_log(__FUNCTION__ . "Could not open directory at: $directory_base_path");
        return false;
    }else{
        while(false !== ($filename = readdir($folder_handle))) {
            if(!in_array($filename, $exclude_array)) {
                if(is_dir($directory_base_path . $filename . "/")) {
                    if($recursive && strcmp($filename, ".")!=0 && strcmp($filename, "..")!=0 ){ // prevent infinite recursion
                        error_log($directory_base_path . $filename . "/");
                        $result_list[$filename] = directory_list("$directory_base_path$filename/", $filter_dir, $filter_files, $exclude, $recursive);
                    }elseif(!$filter_dir){
                        $result_list[] = $filename;
                    }
                }elseif(!$filter_files){
                    $result_list[] = $filename;
                }
            }
        }
        closedir($folder_handle);
        return $result_list;
    }
}



/**
 * This funciton gives the field 'id', which is for every table similar
 * an unique name. This function adds the table name in front of 
 * the string 'id'. This prevents errors during sql queries.
 * 
 * @global type $la_id
 * @param type $field
 * @param type $table_or_relation
 * @return type 
 */
function add_table_name_to_id($field, $table_or_relation)
{
    
    if ($field == 'id' or $field == 'ID') {

        // Remove empty space from relation
        $table = trim($table_or_relation);

        // Convert relation-strint to an array
        $arr_table = explode(' ', $table);

        // Convert table name to an array
        $arr_table_name = explode('_', $arr_table[0]);

        // Ad the name of the table to the field name f.i. addressID
        if (isset($arr_table_name[2])) {
            $added_field = $arr_table_name[2] . 'ID';
        }else{
            $added_field = $arr_table[0] . 'ID';
        }
        
    // Other fields
    }else{
        $added_field = $field;
    }
    
    return $added_field;
}


/**
 *
 * @return boolean 
 */
function check_mobile() {
  $agents = array(
    'Windows CE', 'Pocket', 'Mobile',
    'Portable', 'Smartphone', 'SDA',
    'PDA', 'Handheld', 'Symbian',
    'WAP', 'Palm', 'Avantgo',
    'cHTML', 'BlackBerry', 'Opera Mini',
    'Nokia','Opera Mobi'
  );

  // Prfen der Browserkennung
  for ($i=0; $i<count($agents); $i++) {
    if(isset($_SERVER["HTTP_USER_AGENT"]) && strpos($_SERVER["HTTP_USER_AGENT"], $agents[$i]) !== false)
      return true;
  }

  return false;
}


/**
 * Sets a client id for an address
 * 
 */
function set_client_id()
{
    global $la_client_no_created;
    global $tbl_cli;
    global $tbl_adr;
    global $db;
    global $use;
    
    if ( isset($_REQUEST['action']) and $_REQUEST['action'] == 'set_client_no' ) 
    {
        $db->select_db($use['db_e']);
            // insert new record
            $sql = "
                INSERT INTO $tbl_cli (id_tbl_address)
                VALUE(".$_REQUEST['id'] .")";
            $db->do_sql($sql);

            // write id in field client_no
            $sql = "
                UPDATE $tbl_cli
                SET client_no = LAST_INSERT_ID() 
                WHERE id = LAST_INSERT_ID()";
            $db->do_sql($sql);

            // insert client-id into table address
            $sql = "
                UPDATE $tbl_adr
                SET id_tbl_client = LAST_INSERT_ID() 
                WHERE id = " . $_REQUEST['id'];
            $db->do_sql($sql);
            
            $info = new Info_Box();
            $info->show_box($la_client_no_created);
    }
}


/**
 * initial variable in the session
 *
 * @param type $array
 * @param type $initial_value 
 */
function initial_var($array, $initial_value = '') 
{
    foreach ($array as $value) {
        if (!isset($_SESSION[$value])) {
            $_SESSION[$value] = $initial_value;
        }
    }
}


/**
 * initial variable in the session
 *
 * @param type $array
 * @param type $initial_value 
 */
function print_empty_paragraph($i = 1) 
{
    for ($j=0; $j<$i;$j++) {
      print '<p></p>';
    }
}


/**
 * Put request values to the session
 * 
 * The problem is: you can submit values by $_GET an $_POST only to the next
 * page. On the page after the next page the value doesn't extist.
 * This class checks if a parameter is set by $_GET or $_POST.
 * If this value exist, the class creats a $_SESSION index with the 
 * according value;
 *
 * For example: $_GET['abc'] will create $_SESSION['abc'] with the same value.
 * 
 * Required: A session
 * 
 * @since  2012-02
 */
function request_to_session() 
{
    foreach($_REQUEST as $key => $value) {
        
        /**
         * Check id don't has to be changed, else forms won't be submitted once
         * Don't change security informations
         */
        
        if 
            (
                    $key != 'check_id'
                and $key != 'docum'
                and $key != 'user'
                and $key != 'use_id'
                and $key != 'bea_id'
                and $key != 'bea_username'
             ) 
        {
            if (is_string($value)) 
            {
                $_SESSION[$key] = urldecode($value);
            }
        }
    }
}


/**
 * save session parameters to db
 *
 * @global  $tbl_wor
 * @global  $db
 * @global <type> $tbl_wor
 */
function save_session() 
{
    global $tbl_wor;
    global $db;

    // Session parameters for saving
    $arr = array(
        'auf_id', 
        'year',
        'product_counter',
        );

    // Makes a new array with parameters for saving
    foreach ($arr as  $value) {
        if (isset($_SESSION[$value])) {
            $arr2[$value] = $_SESSION[$value];
        }else{
            $arr2['x'] = '';
        }
    }

    // Convert array to string
    $session_string = json_encode($arr2);

    // Save parameter string to field session in table bearbeiter
    $sql = "
        UPDATE $tbl_wor
        SET session_value = '$session_string'
        WHERE id = " . $_SESSION['bea_id'];
    $db->do_sql($sql);
}


/**
 * this function checks if the personal user dir exist,
 * if not creating user dir
 *
 * @param string $modus p=whole path, d=only directory
 * @return string user directory
 */
function make_user_dir($modus='p') 
{
    global $use;
    $pass = substr($use['pw'], 0, 8);
    
    if (LOCAL_SYSTEM == false) {
        if ($modus == 'p') {
            $folder = PDF_ARCHIV_DIR . '/' . $use['id'] . '_' . $pass . '/';
        }elseif($modus == 'd') {
            $folder =  $use['id'] . '_' . $pass . '/';
        }
    }else{
        /**
         * @todo local system? 
         */
        $folder = PDF_ARCHIV_DIR . '/' ;
        
    }
    if (!file_exists($folder)) {
        mkdir($folder);
    }
    return $folder;
}


/**
 * create html input check id, the timestamp
 * this field prevents double submit of formulars
 */
function check_id_field() 
{
    ?>
    <input type="hidden" name="check_id" value="<?php print time(); ?>">
    <?php
}


/**
 * validate value of font-size
 * 
 * @param <type> $size
 * @return int Font Size
 */
function proof_font_size($size) 
{
    if ($size > 5 and $size < 25) {
        $font_size = $size;
        return $font_size;
    }else{
        $font_size = 10;
        return $font_size;
    }
}


/**
 * Formats date values
 *
 * @param type $date
 * @return string and formatted html string
 */
function show_date($date)
{
    
    // format date
        if ($date == '0000-00-00' or $date == null) {
            // empty date
             return '<span style="width:30px;">--</span>';
             
        }else{
            #$datetime = new DateTime();
            #$datetime->getTimestamp();
            return $date;
        }
        
}



/**
 * display the word "deleded" in red font-color
 */
function show_word_deleted($hide)
{
    global $la_hidden;
    
    if($hide == 1 ) {
        return '<span class="red">' . $la_hidden . '</span>';
    }
}


/**
 * Link back
 * Requires: $GLOBALS['la_back']
 *
 * @param type $html_tag
 * @param type $class
 * @return string in html
 */
function link_back_with_query($html_tag='span', $class='action')
{
    // Create class-part of link
    if (empty($class)) {
        $html_str_class = '';
    }else{
        $html_str_class = 'class="' . $class . '"';
    }
    
    // Http referer if possible
    if (isset ($_SERVER['HTTP_REFERER'])) {
        $href = $_SERVER['HTTP_REFERER'];
    }else{
        $href = "javascript:history.go(-1);";
    }
    // Assemble link
    $link = '<' . $html_tag . '>';
    $link .= '<a ' . $html_str_class . ' href="' . $href . '">' . $GLOBALS['la_back'] . '</a>';
    $link .= '</' . $html_tag . '>';
    return $link;
}


/**
 * Creates select field with automatic submit
 * 
 * @param array $select_array
 * @param string $select_name
 * @param integer $width
 * @param boolean $active
 * @param string $class
 * @return string 
 */
function select_aus_array_auto($select_array, $select_name, $width='100', 
        $active=1, $class='')
{
    if ($active == 1) {
        $onchange = ' onchange="this.form.submit(); ';
        $style = 'style="width:' . $width . 'px;"';
        $title = 'Print documents';
    }else{
        $onchange = '';
        $style = 'style="width:' . $width . 'px; color:silver"';
        $title = 'Modul not active.';
    }
    $output = '<select class="' . $class . '" ' . $style . ' title="' . $title . '" name="' 
        . $select_name . '"' . $onchange . '">' . "\n";
    foreach ($select_array as $key => $value) {
        // preselection
        if(isset ($_SESSION[$select_name]) and $value == $_SESSION[$select_name]) {
            $selected = 'selected';
            #$selected = " ";
        }else{
            $selected = '';
        }
        $output .= '<option class="big" '. $selected . ' value="' . $value 
                . '"> ' . $key . '</option>' . "\n";
    }
    $output .= '</select>';
    return $output;
}


/**
 * Displays a form for searching
 * 
 * @global string $la_search
 * @param string $target
 * @param bool $select_deliverer 
 */
function search_form($target='', $select_deliverer=0) 
{
    global $la_search;
    if (empty($arget)) {
        $target = basename($_SERVER['PHP_SELF']);
    }
    $output = '<form name="such" method="post" action="' . $target . '" style="display:inline">'
            . '<input type="hidden" name="tbl" value="' . $_SESSION['tbl'] . '" >';
    if ($select_deliverer == 1) {
        $output .= '<input type="hidden" name="select_deliverer" value="1" >';
    } 
    $output .= '<input type="hidden" name="search_mod" value="1" >'
            . '<input type="hidden" name="column1" value="alle" >'
            . '<input type="hidden" name="page" value="1">'
            . '<input type="text" name="value1" style="width:12%;" value="';
    if (isset($_SESSION['value1'])) {
        $output .= $_SESSION['value1'];
    }
    $output .= '" onfocus="select()" >'
            . '<input type="submit" name="search_button" value="'. $la_search . ' ... " >'
            . '</form>';
    
    return $output;
}


/**
 * Creates a form for selecting from different tables, 
 * like: order_items, price, ...
 * 
 * @param str $src_file
 * @param str $quell_tbl_vorhandene_posi
 * @param str $ziel_tbl
 * @param str $item
 */
function product_select_form($src_file, $ziel_tbl, $item, $kun_id=3) 
{
    global $la_products;
    global $la_price;
    global $la_product_linked;
    global $la_product_search_and_insert;
    global $use;

        $art = '';
        $pre = '';
        $auf_pos = '';
        $pri_pub = '';
        $kun_art = '';
        
    if (isset($_SESSION['src_tbl']) and $_SESSION['src_tbl'] == 'pro') {
        $art = 'selected';
    }  elseif (isset($_SESSION['src_tbl']) and $_SESSION['src_tbl'] == 'pri') {
        $pre = 'selected';
    }  elseif (isset($_SESSION['src_tbl']) and $_SESSION['src_tbl'] == 'ord_ite') {
        $auf_pos = 'selected';
    }  elseif (isset($_SESSION['src_tbl']) and $_SESSION['src_tbl'] == 'pri_pub') {
        $pri_pub = 'selected';
    }  elseif (isset($_SESSION['src_tbl']) and $_SESSION['src_tbl'] == 'cli_pro') {
        $kun_art = 'selected';
    }
    
    ?>
    <form method="post" action="<?php print PROJECT_HTTP_ROOT; ?>article_select.php">
    <select name="src_tbl">
    <option <?php print $art; ?> value="pro"><?php print $la_products; ?></option>
    <?php
    if (is_dir(PROJECT_DOCUMENT_ROOT.'/modul/label') and $use['mod_label'] == 1) {
        ?>
        <option <?php print $pre; ?> value="pri"><?php print $la_price; ?></option>
        <?php
    } 
    // Q U E R Y //////////////////////////////
    if ($use['id'] < 3 or $use['id'] == 18) {
        ?>
        <option <?php print $auf_pos; ?> value="ord_ite">ch:alle Auftrags-Positionen</option>
        <option <?php print $kun_art; ?> value="cli_pro"><?php print 'ch:'.$la_product_linked; ?></option>
        <?php
    }

    if (LOCAL_SYSTEM == false) {
        ?>
        <option <?php print $pri_pub; ?> value="pri_pub">schreinerwissen-Preise</option>
        <?php
    }
    ?>
        
    </select>

    <input type="hidden" name="page" value="1">
    <input type="hidden" name="column1" value="alle">
    <input type="hidden" name="src_file" value="<?php print $src_file; ?>">
    <input type="hidden" name="target_tbl" value="<?php print $ziel_tbl; ?>">
    <input type="hidden" name="item" value="<?php print $item; ?>">
    <input type="hidden" name="kun_id" value="<?php print $kun_id; ?>">
    <input type="hidden"  value="<?php print $kun_id; ?>">
    <input type="text" name="value1" size="18" value="">
    <input type="submit" name="insert_data" value="<?php print $la_product_search_and_insert; ?>">
    <?php 
    if (isset($_SESSION['tbl']) and $_SESSION['tbl'] == 'cal_mat') {
        ?>
        <input type="checkbox" checked="checked" name="qm" value="1">m und qm
        <?php
    }
        ?>
    </form>
    <?php
}


/**
 * Delete empty array elements
 * @param <type> $values
 */
function unset_empty_array_elements($values) 
{
    for ($i=0 ; $i<=count($values); $i++) {
        if (empty($values[$i])) {
        unset ( $values[$i] );
        }
    }
}


/**
 * Shows empty cells, except the field $pos_feld, 
 * there will be a readonly input field
 * 
 * @param int $quantity Number of columns
 * @param int $position_input 
 */
function show_empty_row($quantity, $position_input) 
{
    print '<tr>';
    for($i=1; $i<=$quantity; $i++) {
        print '<td> &nbsp;';
        if($i == $position_input) {
            print '<input style="width:100%" type="text" readonly="readonly" 
                 value="">';
        }
        print '</td>';
    }
    print '<tr>';
}



/**
 * Returns the input or output array in file tbl_setting.inc.php
 * 
 * @param <type> $param = The short form of the table, f.i. 'ord' like 
 * 'Auftrag'
 * @return var variable without "$"
 */
function get_input_output_array($param) 
{
    $arr = $_SESSION['tbl'] . '_fel_' . $param;
    if (isset($GLOBALS[$arr])) {
        return $GLOBALS[$arr];
        
    }else{
        return '*';
    }
}


/**
 * Gets form of address, f.i. Sehr geehrter Herr etc.
 * 
 * @param <type> $id_tbl_salutation
 * @return string
 */
function get_salutation($param) 
{
    global $arr_salutation_complete;
    
    if ($param == 1) {
        return $arr_salutation_complete[1];

    }elseif($param == 2) {
        return $arr_salutation_complete[2];

    }elseif($param == 3) {
        return $arr_salutation_complete[3];

    }elseif($param == 4) {
        return $arr_salutation_complete[4];

    }elseif($param == 5) {
        return $arr_salutation_complete[5];

    }elseif($param == 6) {
        return $arr_salutation_complete[6];

    }elseif($param == 7) {
        return $arr_salutation_complete[7];

    }
      
}


/**
 * Buttons only for tutor
 * 
 * @global type $la_back
 * @global type $la_cancel
 * @global type $la_next
 * @param type $action_forward
 * @param type $action_back 
 */
function forward_back($action_forward, $action_back='javascript:history.go(-1);') 
{
    global $la_back;
    global $la_cancel;
    global $la_next;
    
    for ($i = 0; $i < 2; $i++) {
        if ($i==0) {
           $button = $la_back;
           $action = $action_back;
        }
        elseif($i==1) {
            $button = $la_cancel;
            $action = '../center.php';
        }elseif($i==2) {
            $button = $la_next;
            $action = $action_forward;
        }
        ?>
        <!--weiter-->
        <form method="post" style="display:inline" action="<?php print $action; ?>">
        <input type="submit" name="button" value="<?php print $button; ?>">
        </form>
        <?php
    }
        print '<br/><br/>';
}



/**
 * Display or hide deleted records 
 * 
 * Place the return ($hide) of this function in your query,
 * f.i. "SELECT id, description, sales_price
 *       WHERE id_tbl_client =  $id  $hide"
 *
 * @param string $name_of_hide_field
 * @return string 
 */
function show_or_hide_deleted_records($name_of_hide_field='hide') 
{
    // check if button hide was pressed
    if( isset($_SESSION['hide']) and $_SESSION['hide'] != 1) {
       
        if (is_array($name_of_hide_field)) {
            $hide = '';
            foreach ($name_of_hide_field as $value) {
                $hide .= " AND $value != '1' ";
            }
        }else{
            // check if $hide is already defined
            if(isset ($name_of_hide_field) and $name_of_hide_field == true) {
                $hide = " AND $name_of_hide_field != '1'";

            }elseif(isset ($name_of_hide_field) and $name_of_hide_field == false) {
                $hide = '';

            }else{
                $hide = " AND $name_of_hide_field != '1'";
            }
        }
    }else{
        $hide = '';
    }
    
    return $hide;
}